#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Support for "Auth"-related System Backend Methods."""
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin
from hvac.utils import validate_list_of_strings_param, list_to_comma_delimited
from hvac import exceptions, utils


class Auth(SystemBackendMixin):

    def list_auth_methods(self):
        """List all enabled auth methods.

        Supported methods:
            GET: /sys/auth. Produces: 200 application/json

        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = '/v1/sys/auth'
        return self._adapter.get(
            url=api_path,
        )

    def enable_auth_method(self, method_type, description=None, config=None, plugin_name=None, local=False, path=None, **kwargs):
        """Enable a new auth method.

        After enabling, the auth method can be accessed and configured via the auth path specified as part of the URL.
        This auth path will be nested under the auth prefix.

        Supported methods:
            POST: /sys/auth/{path}. Produces: 204 (empty body)

        :param method_type: The name of the authentication method type, such as "github" or "token".
        :type method_type: str | unicode
        :param description: A human-friendly description of the auth method.
        :type description: str | unicode
        :param config: Configuration options for this auth method. These are the possible values:

            * **default_lease_ttl**: The default lease duration, specified as a string duration like "5s" or "30m".
            * **max_lease_ttl**: The maximum lease duration, specified as a string duration like "5s" or "30m".
            * **audit_non_hmac_request_keys**: Comma-separated list of keys that will not be HMAC'd by audit devices in
              the request data object.
            * **audit_non_hmac_response_keys**: Comma-separated list of keys that will not be HMAC'd by audit devices in
              the response data object.
            * **listing_visibility**: Speficies whether to show this mount in the UI-specific listing endpoint.
            * **passthrough_request_headers**: Comma-separated list of headers to whitelist and pass from the request to
              the backend.
        :type config: dict
        :param plugin_name: The name of the auth plugin to use based from the name in the plugin catalog. Applies only
            to plugin methods.
        :type plugin_name: str | unicode
        :param local: <Vault enterprise only> Specifies if the auth method is a local only. Local auth methods are not
            replicated nor (if a secondary) removed by replication.
        :type local: bool
        :param path: The path to mount the method on. If not provided, defaults to the value of the "method_type"
            argument.
        :type path: str | unicode
        :param kwargs: All dicts are accepted and passed to vault. See your specific secret engine for details on which
            extra key-word arguments you might want to pass.
        :type kwargs: dict
        :return: The response of the request.
        :rtype: requests.Response
        """
        if path is None:
            path = method_type

        params = {
            'type': method_type,
        }
        params.update(
            utils.remove_nones({
                'description': description,
                'config': config,
                'plugin_name': plugin_name,
                'local': local,
            })
        )
        params.update(kwargs)
        api_path = utils.format_url('/v1/sys/auth/{path}', path=path)
        return self._adapter.post(
            url=api_path,
            json=params
        )

    def disable_auth_method(self, path):
        """Disable the auth method at the given auth path.

        Supported methods:
            DELETE: /sys/auth/{path}. Produces: 204 (empty body)

        :param path: The path the method was mounted on. If not provided, defaults to the value of the "method_type"
            argument.
        :type path: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url('/v1/sys/auth/{path}', path=path)
        return self._adapter.delete(
            url=api_path,
        )

    def read_auth_method_tuning(self, path):
        """Read the given auth path's configuration.

        This endpoint requires sudo capability on the final path, but the same functionality can be achieved without
        sudo via sys/mounts/auth/[auth-path]/tune.

        Supported methods:
            GET: /sys/auth/{path}/tune. Produces: 200 application/json

        :param path: The path the method was mounted on. If not provided, defaults to the value of the "method_type"
            argument.
        :type path: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            '/v1/sys/auth/{path}/tune',
            path=path,
        )
        return self._adapter.get(
            url=api_path,
        )

    def tune_auth_method(self, path, default_lease_ttl=None, max_lease_ttl=None, description=None,
                         audit_non_hmac_request_keys=None, audit_non_hmac_response_keys=None, listing_visibility=None,
                         passthrough_request_headers=None, **kwargs):
        """Tune configuration parameters for a given auth path.

        This endpoint requires sudo capability on the final path, but the same functionality can be achieved without
        sudo via sys/mounts/auth/[auth-path]/tune.

        Supported methods:
            POST: /sys/auth/{path}/tune. Produces: 204 (empty body)

        :param path: The path the method was mounted on. If not provided, defaults to the value of the "method_type"
            argument.
        :type path: str | unicode
        :param default_lease_ttl: Specifies the default time-to-live. If set on a specific auth path, this overrides the
            global default.
        :type default_lease_ttl: int
        :param max_lease_ttl: The maximum time-to-live. If set on a specific auth path, this overrides the global
            default.
        :type max_lease_ttl: int
        :param description: Specifies the description of the mount. This overrides the current stored value, if any.
        :type description: str | unicode
        :param audit_non_hmac_request_keys: Specifies the list of keys that will not be HMAC'd by audit devices in the
            request data object.
        :type audit_non_hmac_request_keys: array
        :param audit_non_hmac_response_keys: Specifies the list of keys that will not be HMAC'd by audit devices in the
            response data object.
        :type audit_non_hmac_response_keys: list
        :param listing_visibility: Specifies whether to show this mount in the UI-specific listing endpoint. Valid
            values are "unauth" or "".
        :type listing_visibility: list
        :param passthrough_request_headers: List of headers to whitelist and pass from the request to the backend.
        :type passthrough_request_headers: list
        :param kwargs: All dicts are accepted and passed to vault. See your specific secret engine for details on which
            extra key-word arguments you might want to pass.
        :type kwargs: dict
        :return: The response of the request.
        :rtype: requests.Response
        """

        if listing_visibility is not None and listing_visibility not in ['unauth', '']:
            error_msg = 'invalid listing_visibility argument provided: "{arg}"; valid values: "unauth" or ""'.format(
                arg=listing_visibility,
            )
            raise exceptions.ParamValidationError(error_msg)

        # All parameters are optional for this method. Until/unless we include input validation, we simply loop over the
        # parameters and add which parameters are set.
        optional_parameters = {
            'default_lease_ttl': {},
            'max_lease_ttl': {},
            'description': {},
            'audit_non_hmac_request_keys': dict(comma_delimited_list=True),
            'audit_non_hmac_response_keys': dict(comma_delimited_list=True),
            'listing_visibility': {},
            'passthrough_request_headers': dict(comma_delimited_list=True),
        }
        params = {}
        for optional_parameter, parameter_specification in optional_parameters.items():
            if locals().get(optional_parameter) is not None:
                if parameter_specification.get('comma_delimited_list'):
                    argument = locals().get(optional_parameter)
                    validate_list_of_strings_param(optional_parameter, argument)
                    params[optional_parameter] = list_to_comma_delimited(argument)
                else:
                    params[optional_parameter] = locals().get(optional_parameter)
        params.update(kwargs)
        api_path = utils.format_url('/v1/sys/auth/{path}/tune', path=path)
        return self._adapter.post(
            url=api_path,
            json=params,
        )



from hvac import utils
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin


class Mount(SystemBackendMixin):

    def list_mounted_secrets_engines(self):
        """Lists all the mounted secrets engines.

        Supported methods:
            POST: /sys/mounts. Produces: 200 application/json

        :return: JSON response of the request.
        :rtype: dict
        """
        return self._adapter.get('/v1/sys/mounts')

    def retrieve_mount_option(self, mount_point, option_name, default_value=None):
        secrets_engine_path = '{mount_point}/'.format(mount_point=mount_point)
        secrets_engines_list = self.list_mounted_secrets_engines()['data']
        mount_options = secrets_engines_list[secrets_engine_path].get('options')
        if mount_options is None:
            return default_value

        return mount_options.get(option_name, default_value)

    def enable_secrets_engine(self, backend_type, path=None, description=None, config=None, plugin_name=None,
                              options=None, local=False, seal_wrap=False, **kwargs):
        """Enable a new secrets engine at the given path.

        Supported methods:
            POST: /sys/mounts/{path}. Produces: 204 (empty body)

        :param backend_type: The name of the backend type, such as "github" or "token".
        :type backend_type: str | unicode
        :param path: The path to mount the method on. If not provided, defaults to the value of the "backend_type"
            argument.
        :type path: str | unicode
        :param description: A human-friendly description of the mount.
        :type description: str | unicode
        :param config: Configuration options for this mount. These are the possible values:

            * **default_lease_ttl**: The default lease duration, specified as a string duration like "5s" or "30m".
            * **max_lease_ttl**: The maximum lease duration, specified as a string duration like "5s" or "30m".
            * **force_no_cache**: Disable caching.
            * **plugin_name**: The name of the plugin in the plugin catalog to use.
            * **audit_non_hmac_request_keys**: Comma-separated list of keys that will not be HMAC'd by audit devices in
              the request data object.
            * **audit_non_hmac_response_keys**: Comma-separated list of keys that will not be HMAC'd by audit devices in
              the response data object.
            * **listing_visibility**: Specifies whether to show this mount in the UI-specific listing endpoint. ("unauth" or "hidden")
            * **passthrough_request_headers**: Comma-separated list of headers to whitelist and pass from the request to
              the backend.
        :type config: dict
        :param options: Specifies mount type specific options that are passed to the backend.

            * **version**: <KV> The version of the KV to mount. Set to "2" for mount KV v2.
        :type options: dict
        :param plugin_name: Specifies the name of the plugin to use based from the name in the plugin catalog. Applies only to plugin backends.
        :type plugin_name: str | unicode
        :param local: <Vault enterprise only> Specifies if the auth method is a local only. Local auth methods are not
            replicated nor (if a secondary) removed by replication.
        :type local: bool
        :param seal_wrap: <Vault enterprise only> Enable seal wrapping for the mount.
        :type seal_wrap: bool
        :param kwargs: All dicts are accepted and passed to vault. See your specific secret engine for details on which
            extra key-word arguments you might want to pass.
        :type kwargs: dict
        :return: The response of the request.
        :rtype: requests.Response
        """
        if path is None:
            path = backend_type

        params = {
            'type': backend_type,
            'description': description,
            'config': config,
            'options': options,
            'plugin_name': plugin_name,
            'local': local,
            'seal_wrap': seal_wrap,
        }

        params.update(kwargs)

        api_path = utils.format_url('/v1/sys/mounts/{path}', path=path)
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def disable_secrets_engine(self, path):
        """Disable the mount point specified by the provided path.

        Supported methods:
            DELETE: /sys/mounts/{path}. Produces: 204 (empty body)

        :param path: Specifies the path where the secrets engine will be mounted. This is specified as part of the URL.
        :type path: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url('/v1/sys/mounts/{path}', path=path)
        return self._adapter.delete(
            url=api_path,
        )

    def read_mount_configuration(self, path):
        """Read the given mount's configuration.

        Unlike the mounts endpoint, this will return the current time in seconds for each TTL, which may be the system
        default or a mount-specific value.

        Supported methods:
            GET: /sys/mounts/{path}/tune. Produces: 200 application/json

        :param path: Specifies the path where the secrets engine will be mounted. This is specified as part of the URL.
        :type path: str | unicode
        :return: The JSON response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url('/v1/sys/mounts/{path}/tune', path=path)
        return self._adapter.get(
            url=api_path,
        )

    def tune_mount_configuration(self, path, default_lease_ttl=None, max_lease_ttl=None, description=None,
                                 audit_non_hmac_request_keys=None, audit_non_hmac_response_keys=None,
                                 listing_visibility=None, passthrough_request_headers=None, options=None, force_no_cache=None, **kwargs):
        """Tune configuration parameters for a given mount point.

        Supported methods:
            POST: /sys/mounts/{path}/tune. Produces: 204 (empty body)

        :param path: Specifies the path where the secrets engine will be mounted. This is specified as part of the URL.
        :type path: str | unicode
        :param mount_point: The path the associated secret backend is mounted
        :type mount_point: str
        :param description: Specifies the description of the mount. This overrides the current stored value, if any.
        :type description: str
        :param default_lease_ttl: Default time-to-live. This overrides the global default. A value of 0 is equivalent to
            the system default TTL
        :type default_lease_ttl: int
        :param max_lease_ttl: Maximum time-to-live. This overrides the global default. A value of 0 are equivalent and
            set to the system max TTL.
        :type max_lease_ttl: int
        :param audit_non_hmac_request_keys: Specifies the comma-separated list of keys that will not be HMAC'd by audit
            devices in the request data object.
        :type audit_non_hmac_request_keys: list
        :param audit_non_hmac_response_keys: Specifies the comma-separated list of keys that will not be HMAC'd by audit
            devices in the response data object.
        :type audit_non_hmac_response_keys: list
        :param listing_visibility: Speficies whether to show this mount in the UI-specific listing endpoint. Valid
            values are "unauth" or "".
        :type listing_visibility: str
        :param passthrough_request_headers: Comma-separated list of headers to whitelist and pass from the request
            to the backend.
        :type passthrough_request_headers: str
        :param options: Specifies mount type specific options that are passed to the backend.

            * **version**: <KV> The version of the KV to mount. Set to "2" for mount KV v2.
        :type options: dict
        :param force_no_cache: Disable caching.
        :type force_no_cache: bool
        :param kwargs: All dicts are accepted and passed to vault. See your specific secret engine for details on which
            extra key-word arguments you might want to pass.
        :type kwargs: dict
        :return: The response from the request.
        :rtype: request.Response
        """
        # All parameters are optional for this method. Until/unless we include input validation, we simply loop over the
        # parameters and add which parameters are set.
        optional_parameters = [
            'default_lease_ttl',
            'max_lease_ttl',
            'description',
            'audit_non_hmac_request_keys',
            'audit_non_hmac_response_keys',
            'listing_visibility',
            'passthrough_request_headers',
            'force_no_cache',
            'options',
        ]
        params = {}
        for optional_parameter in optional_parameters:
            if locals().get(optional_parameter) is not None:
                params[optional_parameter] = locals().get(optional_parameter)

        params.update(kwargs)

        api_path = utils.format_url('/v1/sys/mounts/{path}/tune', path=path)
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def move_backend(self, from_path, to_path):
        """Move an already-mounted backend to a new mount point.

        Supported methods:
            POST: /sys/remount. Produces: 204 (empty body)

        :param from_path: Specifies the previous mount point.
        :type from_path: str | unicode
        :param to_path: Specifies the new destination mount point.
        :type to_path: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            'from': from_path,
            'to': to_path,
        }
        api_path = '/v1/sys/remount'
        return self._adapter.post(
            url=api_path,
            json=params,
        )



import hmac
from datetime import datetime
from hashlib import sha256
import requests


class SigV4Auth(object):
    def __init__(self, access_key, secret_key, session_token=None, region='us-east-1'):
        self.access_key = access_key
        self.secret_key = secret_key
        self.session_token = session_token
        self.region = region

    def add_auth(self, request):
        timestamp = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
        request.headers['X-Amz-Date'] = timestamp

        if self.session_token:
            request.headers['X-Amz-Security-Token'] = self.session_token

        # https://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html
        canonical_headers = ''.join('{0}:{1}\n'.format(k.lower(), request.headers[k]) for k in sorted(request.headers))
        signed_headers = ';'.join(k.lower() for k in sorted(request.headers))
        payload_hash = sha256(request.body.encode('utf-8')).hexdigest()
        canonical_request = '\n'.join([request.method, '/', '', canonical_headers, signed_headers, payload_hash])

        # https://docs.aws.amazon.com/general/latest/gr/sigv4-create-string-to-sign.html
        algorithm = 'AWS4-HMAC-SHA256'
        credential_scope = '/'.join([timestamp[0:8], self.region, 'sts', 'aws4_request'])
        canonical_request_hash = sha256(canonical_request.encode('utf-8')).hexdigest()
        string_to_sign = '\n'.join([algorithm, timestamp, credential_scope, canonical_request_hash])

        # https://docs.aws.amazon.com/general/latest/gr/sigv4-calculate-signature.html
        key = 'AWS4{0}'.format(self.secret_key).encode('utf-8')
        key = hmac.new(key, timestamp[0:8].encode('utf-8'), sha256).digest()
        key = hmac.new(key, self.region.encode('utf-8'), sha256).digest()
        key = hmac.new(key, 'sts'.encode('utf-8'), sha256).digest()
        key = hmac.new(key, 'aws4_request'.encode('utf-8'), sha256).digest()
        signature = hmac.new(key, string_to_sign.encode('utf-8'), sha256).hexdigest()

        # https://docs.aws.amazon.com/general/latest/gr/sigv4-add-signature-to-request.html
        authorization = '{0} Credential={1}/{2}, SignedHeaders={3}, Signature={4}'.format(
            algorithm, self.access_key, credential_scope, signed_headers, signature)
        request.headers['Authorization'] = authorization


def generate_sigv4_auth_request(header_value=None):
    """Helper function to prepare a AWS API request to subsequently generate a "AWS Signature Version 4" header.

    :param header_value: Vault allows you to require an additional header, X-Vault-AWS-IAM-Server-ID, to be present
        to mitigate against different types of replay attacks. Depending on the configuration of the AWS auth
        backend, providing a argument to this optional parameter may be required.
    :type header_value: str
    :return: A PreparedRequest instance, optionally containing the provided header value under a
        'X-Vault-AWS-IAM-Server-ID' header name pointed to AWS's simple token service with action "GetCallerIdentity"
    :rtype: requests.PreparedRequest
    """
    request = requests.Request(
        method='POST',
        url='https://sts.amazonaws.com/',
        headers={'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8', 'Host': 'sts.amazonaws.com'},
        data='Action=GetCallerIdentity&Version=2011-06-15',
    )

    if header_value:
        request.headers['X-Vault-AWS-IAM-Server-ID'] = header_value

    prepared_request = request.prepare()
    return prepared_request



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the AWS auth method and/or secrets engine."""

DEFAULT_MOUNT_POINT = 'aws'
ALLOWED_CREDS_ENDPOINTS = ['creds', 'sts']
ALLOWED_CREDS_TYPES = ['iam_user', 'assumed_role', 'federation_token']
ALLOWED_IAM_ALIAS_TYPES = ['role_id', 'unique_id', 'full_arn']
ALLOWED_EC2_ALIAS_TYPES = ['role_id', 'instance_id', 'image_id']



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin
from hvac.exceptions import ParamValidationError


class Key(SystemBackendMixin):

    def read_root_generation_progress(self):
        """Read the configuration and process of the current root generation attempt.

        Supported methods:
            GET: /sys/generate-root/attempt. Produces: 200 application/json

        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = '/v1/sys/generate-root/attempt'
        return self._adapter.get(
            url=api_path,
        )

    def start_root_token_generation(self, otp=None, pgp_key=None):
        """Initialize a new root generation attempt.

        Only a single root generation attempt can take place at a time. One (and only one) of otp or pgp_key are
        required.

        Supported methods:
            PUT: /sys/generate-root/attempt. Produces: 200 application/json

        :param otp: Specifies a base64-encoded 16-byte value. The raw bytes of the token will be XOR'd with this value
            before being returned to the final unseal key provider.
        :type otp: str | unicode
        :param pgp_key: Specifies a base64-encoded PGP public key. The raw bytes of the token will be encrypted with
            this value before being returned to the final unseal key provider.
        :type pgp_key: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {}
        if otp is not None and pgp_key is not None:
            raise ParamValidationError('one (and only one) of otp or pgp_key arguments are required')
        if otp is not None:
            params['otp'] = otp
        if pgp_key is not None:
            params['pgp_key'] = pgp_key

        api_path = '/v1/sys/generate-root/attempt'
        return self._adapter.put(
            url=api_path,
            json=params
        )

    def generate_root(self, key, nonce):
        """Enter a single master key share to progress the root generation attempt.

        If the threshold number of master key shares is reached, Vault will complete the root generation and issue the
        new token. Otherwise, this API must be called multiple times until that threshold is met. The attempt nonce must
        be provided with each call.

        Supported methods:
            PUT: /sys/generate-root/update. Produces: 200 application/json

        :param key: Specifies a single master key share.
        :type key: str | unicode
        :param nonce: The nonce of the attempt.
        :type nonce: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {
            'key': key,
            'nonce': nonce,
        }
        api_path = '/v1/sys/generate-root/update'
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def cancel_root_generation(self):
        """Cancel any in-progress root generation attempt.

        This clears any progress made. This must be called to change the OTP or PGP key being used.

        Supported methods:
            DELETE: /sys/generate-root/attempt. Produces: 204 (empty body)

        :return: The response of the request.
        :rtype: request.Response
        """
        api_path = '/v1/sys/generate-root/attempt'
        return self._adapter.delete(
            url=api_path,
        )

    def get_encryption_key_status(self):
        """Read information about the current encryption key used by Vault.

        Supported methods:
            GET: /sys/key-status. Produces: 200 application/json

        :return: JSON response with information regarding the current encryption key used by Vault.
        :rtype: dict
        """
        api_path = '/v1/sys/key-status'
        return self._adapter.get(
            url=api_path,
        )

    def rotate_encryption_key(self):
        """Trigger a rotation of the backend encryption key.

        This is the key that is used to encrypt data written to the storage backend, and is not provided to operators.
        This operation is done online. Future values are encrypted with the new key, while old values are decrypted with
        previous encryption keys.

        This path requires sudo capability in addition to update.

        Supported methods:
            PUT: /sys/rorate. Produces: 204 (empty body)

        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = '/v1/sys/rotate'
        return self._adapter.put(
            url=api_path,
        )

    def read_rekey_progress(self, recovery_key=False):
        """Read the configuration and progress of the current rekey attempt.

        Supported methods:
            GET: /sys/rekey-recovery-key/init. Produces: 200 application/json
            GET: /sys/rekey/init. Produces: 200 application/json

        :param recovery_key: If true, send requests to "rekey-recovery-key" instead of "rekey" api path.
        :type recovery_key: bool
        :return: The JSON response of the request.
        :rtype: requests.Response
        """
        api_path = '/v1/sys/rekey/init'
        if recovery_key:
            api_path = '/v1/sys/rekey-recovery-key/init'
        return self._adapter.get(
            url=api_path,
        )

    def start_rekey(self, secret_shares=5, secret_threshold=3, pgp_keys=None, backup=False, require_verification=False, recovery_key=False):
        """Initializes a new rekey attempt.

        Only a single recovery key rekeyattempt can take place at a time, and changing the parameters of a rekey
        requires canceling and starting a new rekey, which will also provide a new nonce.

        Supported methods:
            PUT: /sys/rekey/init. Produces: 204 (empty body)
            PUT: /sys/rekey-recovery-key/init. Produces: 204 (empty body)

        :param secret_shares: Specifies the number of shares to split the master key into.
        :type secret_shares: int
        :param secret_threshold: Specifies the number of shares required to reconstruct the master key. This must be
            less than or equal to secret_shares.
        :type secret_threshold: int
        :param pgp_keys: Specifies an array of PGP public keys used to encrypt the output unseal keys. Ordering is
            preserved. The keys must be base64-encoded from their original binary representation. The size of this array
            must be the same as secret_shares.
        :type pgp_keys: list
        :param backup: Specifies if using PGP-encrypted keys, whether Vault should also store a plaintext backup of the
            PGP-encrypted keys at core/unseal-keys-backup in the physical storage backend. These can then be retrieved
            and removed via the sys/rekey/backup endpoint.
        :type backup: bool
        :param require_verification: This turns on verification functionality. When verification is turned on, after
            successful authorization with the current unseal keys, the new unseal keys are returned but the master key
            is not actually rotated. The new keys must be provided to authorize the actual rotation of the master key.
            This ensures that the new keys have been successfully saved and protects against a risk of the keys being
            lost after rotation but before they can be persisted. This can be used with without pgp_keys, and when used
            with it, it allows ensuring that the returned keys can be successfully decrypted before committing to the
            new shares, which the backup functionality does not provide.
        :param recovery_key: If true, send requests to "rekey-recovery-key" instead of "rekey" api path.
        :type recovery_key: bool
        :type require_verification: bool
        :return: The JSON dict of the response.
        :rtype: dict | request.Response
        """
        params = {
            'secret_shares': secret_shares,
            'secret_threshold': secret_threshold,
            'require_verification': require_verification,
        }

        if pgp_keys:
            if len(pgp_keys) != secret_shares:
                raise ParamValidationError('length of pgp_keys argument must equal secret shares value')

            params['pgp_keys'] = pgp_keys
            params['backup'] = backup

        api_path = '/v1/sys/rekey/init'
        if recovery_key:
            api_path = '/v1/sys/rekey-recovery-key/init'
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def cancel_rekey(self, recovery_key=False):
        """Cancel any in-progress rekey.

        This clears the rekey settings as well as any progress made. This must be called to change the parameters of the
        rekey.

        Note: Verification is still a part of a rekey. If rekeying is canceled during the verification flow, the current
        unseal keys remain valid.

        Supported methods:
            DELETE: /sys/rekey/init. Produces: 204 (empty body)
            DELETE: /sys/rekey-recovery-key/init. Produces: 204 (empty body)

        :param recovery_key: If true, send requests to "rekey-recovery-key" instead of "rekey" api path.
        :type recovery_key: bool
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = '/v1/sys/rekey/init'
        if recovery_key:
            api_path = '/v1/sys/rekey-recovery-key/init'
        return self._adapter.delete(
            url=api_path,
        )

    def rekey(self, key, nonce=None, recovery_key=False):
        """Enter a single recovery key share to progress the rekey of the Vault.

        If the threshold number of recovery key shares is reached, Vault will complete the rekey. Otherwise, this API
        must be called multiple times until that threshold is met. The rekey nonce operation must be provided with each
        call.

        Supported methods:
            PUT: /sys/rekey/update. Produces: 200 application/json
            PUT: /sys/rekey-recovery-key/update. Produces: 200 application/json

        :param key: Specifies a single recovery share key.
        :type key: str | unicode
        :param nonce: Specifies the nonce of the rekey operation.
        :type nonce: str | unicode
        :param recovery_key: If true, send requests to "rekey-recovery-key" instead of "rekey" api path.
        :type recovery_key: bool
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {
            'key': key,
        }

        if nonce is not None:
            params['nonce'] = nonce

        api_path = '/v1/sys/rekey/update'
        if recovery_key:
            api_path = '/v1/sys/rekey-recovery-key/update'
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def rekey_multi(self, keys, nonce=None, recovery_key=False):
        """Enter multiple recovery key shares to progress the rekey of the Vault.

        If the threshold number of recovery key shares is reached, Vault will complete the rekey.

        :param keys: Specifies multiple recovery share keys.
        :type keys: list
        :param nonce: Specifies the nonce of the rekey operation.
        :type nonce: str | unicode
        :param recovery_key: If true, send requests to "rekey-recovery-key" instead of "rekey" api path.
        :type recovery_key: bool
        :return: The last response of the rekey request.
        :rtype: response.Request
        """
        result = None

        for key in keys:
            result = self.rekey(
                key=key,
                nonce=nonce,
                recovery_key=recovery_key,
            )
            if result.get('complete'):
                break

        return result

    def read_backup_keys(self, recovery_key=False):
        """Retrieve the backup copy of PGP-encrypted unseal keys.

        The returned value is the nonce of the rekey operation and a map of PGP key fingerprint to hex-encoded
        PGP-encrypted key.

        Supported methods:
            PUT: /sys/rekey/backup. Produces: 200 application/json
            PUT: /sys/rekey-recovery-key/backup. Produces: 200 application/json

        :param recovery_key: If true, send requests to "rekey-recovery-key" instead of "rekey" api path.
        :type recovery_key: bool
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = '/v1/sys/rekey/backup'
        if recovery_key:
            api_path = '/v1/sys/rekey-recovery-key/backup'
        return self._adapter.get(
            url=api_path,
        )

    def cancel_rekey_verify(self):
        """Cancel any in-progress rekey verification.
        This clears any progress made and resets the nonce. Unlike cancel_rekey, this only resets
        the current verification operation, not the entire rekey atttempt.
        The return value is the same as GET along with the new nonce.

        Supported methods:
            DELETE: /sys/rekey/verify. Produces: 204 (empty body)

        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = '/v1/sys/rekey/verify'
        return self._adapter.delete(
            url=api_path,
        )

    def rekey_verify(self, key, nonce):
        """Enter a single new recovery key share to progress the rekey verification of the Vault.
        If the threshold number of new recovery key shares is reached, Vault will complete the
        rekey. Otherwise, this API must be called multiple times until that threshold is met.
        The rekey verification nonce must be provided with each call.

        Supported methods:
            PUT: /sys/rekey/verify. Produces: 200 application/json

        :param key: Specifies multiple recovery share keys.
        :type key: str | unicode
        :param nonce: Specifies the nonce of the rekey verify operation.
        :type nonce: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {
            'key': key,
            'nonce': nonce,
        }

        api_path = '/v1/sys/rekey/verify'
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def rekey_verify_multi(self, keys, nonce):
        """Enter multiple new recovery key shares to progress the rekey verification of the Vault.
        If the threshold number of new recovery key shares is reached, Vault will complete the
        rekey. Otherwise, this API must be called multiple times until that threshold is met.
        The rekey verification nonce must be provided with each call.

        Supported methods:
            PUT: /sys/rekey/verify. Produces: 200 application/json

        :param keys: Specifies multiple recovery share keys.
        :type keys: list
        :param nonce: Specifies the nonce of the rekey verify operation.
        :type nonce: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        result = None

        for key in keys:
            result = self.rekey_verify(
                key=key,
                nonce=nonce,
            )
            if result.get('complete'):
                break

        return result

    def read_rekey_verify_progress(self):
        """Read the configuration and progress of the current rekey verify attempt.

        Supported methods:
            GET: /sys/rekey/verify. Produces: 200 application/json

        :return: The JSON response of the request.
        :rtype: requests.Response
        """
        api_path = '/v1/sys/rekey/verify'
        return self._adapter.get(
            url=api_path,
        )



from hvac import utils
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin


class Namespace(SystemBackendMixin):

    def create_namespace(self, path):
        """Create a namespace at the given path.

        Supported methods:
            POST: /sys/namespaces/{path}. Produces: 200 application/json

        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url('/v1/sys/namespaces/{path}', path=path)
        return self._adapter.post(
            url=api_path,
        )

    def list_namespaces(self):
        """Lists all the namespaces.

        Supported methods:
            LIST: /sys/namespaces. Produces: 200 application/json

        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = '/v1/sys/namespaces/'
        return self._adapter.list(
            url=api_path,
        )

    def delete_namespace(self, path):
        """Delete a namespaces. You cannot delete a namespace with existing child namespaces.

        Supported methods:
            DELETE: /sys/namespaces. Produces: 204 (empty body)

        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url('/v1/sys/namespaces/{path}', path=path)
        return self._adapter.delete(
            url=api_path,
        )



class VaultError(Exception):
    def __init__(self, message=None, errors=None, method=None, url=None):
        if errors:
            message = ', '.join(errors)

        self.errors = errors
        self.method = method
        self.url = url

        super(VaultError, self).__init__(message)

    def __str__(self):
        return "{0}, on {1} {2}".format(self.args[0], self.method, self.url)


class InvalidRequest(VaultError):
    pass


class Unauthorized(VaultError):
    pass


class Forbidden(VaultError):
    pass


class InvalidPath(VaultError):
    pass


class RateLimitExceeded(VaultError):
    pass


class InternalServerError(VaultError):
    pass


class VaultNotInitialized(VaultError):
    pass


class VaultDown(VaultError):
    pass


class UnexpectedError(VaultError):
    pass


class BadGateway(VaultError):
    pass


class ParamValidationError(VaultError):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Transform secrets engine methods module."""
from hvac import utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = 'transform'


class Transform(VaultApiBase):
    """Transform Secrets Engine (API).

    Reference: https://www.vaultproject.io/api-docs/secret/transform
    """

    def create_or_update_role(self, name, transformations, mount_point=DEFAULT_MOUNT_POINT):
        """Creates or update the role with the given name.

        If a role with the name does not exist, it will be created. If the role exists, it will be
        updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/role/:name.

        :param name: the name of the role to create. This is part of the request URL.
        :type name: str | unicode
        :param transformations: Specifies the transformations that can be used with this role.
            At least one transformation is required.
        :type transformations: list
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_role request.
        :rtype: requests.Response
        """
        params = {
            'transformations': transformations,
        }
        api_path = '/v1/{mount_point}/role/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Query an existing role by the given name.

        Supported methods:
            GET: /{mount_point}/role/:name.

        :param name: the name of the role to read. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_role request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/role/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_roles(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing roles in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/role.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_roles request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/role'.format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing role by the given name.

        Supported methods:
            DELETE: /{mount_point}/role/:name.

        :param name: the name of the role to delete. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_role request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/role/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_or_update_transformation(self, name, transform_type, template, tweak_source="supplied",
                                        masking_character="*", allowed_roles=None, mount_point=DEFAULT_MOUNT_POINT):
        """Create or update a transformation with the given name.

        If a transformation with the name does not exist, it will be created. If the
        transformation exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/transformation/:name.

        :param name: the name of the transformation to create or update. This is part of
            the request URL.
        :type name: str | unicode
        :param transform_type: Specifies the type of transformation to perform.
            The types currently supported by this backend are fpe and masking.
            This value cannot be modified by an update operation after creation.
        :type transform_type: str | unicode
        :param template: the template name to use for matching value on encode and decode
            operations when using this transformation.
        :type template: str | unicode
        :param tweak_source: Only used when the type is FPE.
        :type tweak_source: str | unicode
        :param masking_character: the character to use for masking. If multiple characters are
            provided, only the first one is used and the rest is ignored. Only used when
            the type is masking.
        :type masking_character: str | unicode
        :param allowed_roles: a list of allowed roles that this transformation can be assigned to.
            A role using this transformation must exist in this list in order for
            encode and decode operations to properly function.
        :type allowed_roles: list
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_ation request.
        :rtype: requests.Response
        """
        params = {
            'type': transform_type,
            'template': template,
            'tweak_source': tweak_source,
            'masking_character': masking_character,
        }
        params.update(utils.remove_nones({
            'allowed_roles': allowed_roles,
        }))
        api_path = '/v1/{mount_point}/transformation/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_transformation(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Query an existing transformation by the given name.

        Supported methods:
            GET: /{mount_point}/transformation/:name.

        :param name: Specifies the name of the role to read.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_ation request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/transformation/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_transformations(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing transformations in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/transformation.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_ation request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/transformation'.format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_transformation(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing transformation by the given name.

        Supported methods:
            DELETE: /{mount_point}/transformation/:name.

        :param name: the name of the transformation to delete. This is part of the
            request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_ation request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/transformation/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_or_update_template(self, name, template_type, pattern, alphabet, mount_point=DEFAULT_MOUNT_POINT):
        """Creates or update a template with the given name.

        If a template with the name does not exist, it will be created. If the
        template exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/template/:name.

        :param name: the name of the template to create.
        :type name: str | unicode
        :param template_type: Specifies the type of pattern matching to perform.
            The ony type currently supported by this backend is regex.
        :type template_type: str | unicode
        :param pattern: the pattern used to match a particular value. For regex type
            matching, capture group determines the set of character that should be matched
            against. Any matches outside of capture groups are retained
            post-transformation.
        :type pattern: str | unicode
        :param alphabet: the name of the alphabet to use when this template is used for FPE
            encoding and decoding operations.
        :type alphabet: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_template request.
        :rtype: requests.Response
        """
        params = {
            'type': template_type,
            'pattern': pattern,
            'alphabet': alphabet,
        }
        api_path = '/v1/{mount_point}/template/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_template(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Query an existing template by the given name.

        Supported methods:
            GET: /{mount_point}/template/:name.

        :param name: Specifies the name of the role to read.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_template request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/template/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_templates(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing templates in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/transformation.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_template request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/template'.format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_template(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing template by the given name.

        Supported methods:
            DELETE: /{mount_point}/template/:name.

        :param name: the name of the template to delete. This is part of the
            request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_template request.
        :rtype: requests.Response
        """
        params = {
            'name': name,
        }
        api_path = '/v1/{mount_point}/template/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
            json=params,
        )

    def create_or_update_alphabet(self, name, alphabet, mount_point=DEFAULT_MOUNT_POINT):
        """Create or update an alphabet with the given name.

        If an alphabet with the name does not exist, it will be created. If the
        alphabet exists, it will be updated with the new attributes.

        Supported methods:
            POST: /{mount_point}/alphabet/:name.

        :param name: Specifies the name of the transformation alphabet to create.
        :type name: str | unicode
        :param alphabet: the set of characters that can exist within the provided value
            and the encoded or decoded value for a FPE transformation.
        :type alphabet: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_alphabet request.
        :rtype: requests.Response
        """
        params = {
            'alphabet': alphabet,
        }
        api_path = '/v1/{mount_point}/alphabet/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_alphabet(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Queries an existing alphabet by the given name.

        Supported methods:
            GET: /{mount_point}/alphabet/:name.


        :param name: the name of the alphabet to delete. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the read_alphabet request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/alphabet/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_alphabets(self, mount_point=DEFAULT_MOUNT_POINT):
        """List all existing alphabets in the secrets engine.

        Supported methods:
            LIST: /{mount_point}/alphabet.

        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the list_alphabets request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/alphabet'.format(mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_alphabet(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing alphabet by the given name.

        Supported methods:
            DELETE: /{mount_point}/alphabet/:name.

        :param name: the name of the alphabet to delete. This is part of the request URL.
        :type name: str | unicode
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_alphabet request.
        :rtype: requests.Response
        """
        api_path = '/v1/{mount_point}/alphabet/{name}'.format(
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def encode(self, role_name, value=None, transformation=None, tweak=None, batch_input=None, mount_point=DEFAULT_MOUNT_POINT):
        """Encode the provided value using a named role.

        Supported methods:
            POST: /{mount_point}/encode/:role_name.

        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str | unicode
        :param value: the value to be encoded.
        :type value: str | unicode
        :param transformation: the transformation within the role that should be used for this
            encode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str | unicode
        :param tweak: the tweak source.
        :type tweak: str | unicode
        :param batch_input: a list of items to be encoded in a single batch. When this
            parameter is set, the 'value', 'transformation' and 'tweak' parameters are
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: list
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the encode request.
        :rtype: requests.Response
        """
        params = utils.remove_nones({
            'value': value,
            'transformation': transformation,
            'tweak': tweak,
            'batch_input': batch_input,
        })
        api_path = '/v1/{mount_point}/encode/{role_name}'.format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def decode(self, role_name, value=None, transformation=None, tweak=None, batch_input=None, mount_point=DEFAULT_MOUNT_POINT):
        """Decode the provided value using a named role.

        Supported methods:
            POST: /{mount_point}/decode/:role_name.

        :param role_name: the role name to use for this operation. This is specified as part
            of the URL.
        :type role_name: str | unicode
        :param value: the value to be decoded.
        :type value: str | unicode
        :param transformation: the transformation within the role that should be used for this
            decode operation. If a single transformation exists for role, this parameter
            may be skipped and will be inferred. If multiple transformations exist, one
            must be specified.
        :type transformation: str | unicode
        :param tweak: the tweak source.
        :type tweak: str | unicode
        :param batch_input: a list of items to be decoded in a single batch. When this
            parameter is set, the 'value', 'transformation' and 'tweak' parameters are
            ignored. Instead, the aforementioned parameters should be provided within
            each object in the list.
        :type batch_input: array<object>
        :param mount_point: The "path" the secrets engine was mounted on.
        :type mount_point: str | unicode
        :return: The response of the decode request.
        :rtype: requests.Response
        """
        params = utils.remove_nones({
            'value': value,
            'transformation': transformation,
            'tweak': tweak,
            'batch_input': batch_input,
        })
        api_path = '/v1/{mount_point}/decode/{role_name}'.format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the hvac.Client class."""

DEPRECATED_PROPERTIES = {
    'github': dict(
        to_be_removed_in_version='0.9.0',
        client_property='auth',
    ),
    'ldap': dict(
        to_be_removed_in_version='0.9.0',
        client_property='auth',
    ),
    'mfa': dict(
        to_be_removed_in_version='0.9.0',
        client_property='auth',
    ),
    'kv': dict(
        to_be_removed_in_version='0.9.0',
        client_property='secrets',
    ),
}

DEFAULT_URL = 'http://localhost:8200'



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin


class Capabilities(SystemBackendMixin):

    def get_capabilities(self, paths, token=None, accessor=None):
        """Get the capabilities associated with a token.

        Supported methods:
            POST: /sys/capabilities-self. Produces: 200 application/json
            POST: /sys/capabilities. Produces: 200 application/json
            POST: /sys/capabilities-accessor. Produces: 200 application/json

        :param paths: Paths on which capabilities are being queried.
        :type paths: List[str]
        :param token: Token for which capabilities are being queried.
        :type token: str
        :param accessor: Accessor of the token for which capabilities are being queried.
        :type accessor: str
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {
            'paths': paths,
        }

        if token and accessor:
            raise ValueError("You can specify either token or accessor, not both.")
        elif token:
            # https://www.vaultproject.io/api/system/capabilities.html
            params['token'] = token
            api_path = '/v1/sys/capabilities'
        elif accessor:
            # https://www.vaultproject.io/api/system/capabilities-accessor.html
            params['accessor'] = accessor
            api_path = '/v1/sys/capabilities-accessor'
        else:
            # https://www.vaultproject.io/api/system/capabilities-self.html
            api_path = '/v1/sys/capabilities-self'

        return self._adapter.post(
            url=api_path,
            json=params,
        )



from hvac import utils
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin


class Lease(SystemBackendMixin):

    def read_lease(self, lease_id):
        """Retrieve lease metadata.

        Supported methods:
            PUT: /sys/leases/lookup. Produces: 200 application/json

        :param lease_id: the ID of the lease to lookup.
        :type lease_id: str | unicode
        :return: Parsed JSON response from the leases PUT request
        :rtype: dict.
        """
        params = {
            'lease_id': lease_id
        }
        api_path = '/v1/sys/leases/lookup'
        return self._adapter.put(
            url=api_path,
            json=params
        )

    def list_leases(self, prefix):
        """Retrieve a list of lease ids.

        Supported methods:
            LIST: /sys/leases/lookup/{prefix}. Produces: 200 application/json

        :param prefix: Lease prefix to filter list by.
        :type prefix: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url('/v1/sys/leases/lookup/{prefix}', prefix=prefix)
        return self._adapter.list(
            url=api_path,
        )

    def renew_lease(self, lease_id, increment=None):
        """Renew a lease, requesting to extend the lease.

        Supported methods:
            PUT: /sys/leases/renew. Produces: 200 application/json

        :param lease_id: The ID of the lease to extend.
        :type lease_id: str | unicode
        :param increment: The requested amount of time (in seconds) to extend the lease.
        :type increment: int
        :return: The JSON response of the request
        :rtype: dict
        """
        params = {
            'lease_id': lease_id,
            'increment': increment,
        }
        api_path = '/v1/sys/leases/renew'
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def revoke_lease(self, lease_id):
        """Revoke a lease immediately.

        Supported methods:
            PUT: /sys/leases/revoke. Produces: 204 (empty body)

        :param lease_id: Specifies the ID of the lease to revoke.
        :type lease_id: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            'lease_id': lease_id,
        }
        api_path = '/v1/sys/leases/revoke'
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def revoke_prefix(self, prefix):
        """Revoke all secrets (via a lease ID prefix) or tokens (via the tokens' path property) generated under a given
        prefix immediately.

        This requires sudo capability and access to it should be tightly controlled as it can be used to revoke very
        large numbers of secrets/tokens at once.

        Supported methods:
            PUT: /sys/leases/revoke-prefix/{prefix}. Produces: 204 (empty body)


        :param prefix: The prefix to revoke.
        :type prefix: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            'prefix': prefix,
        }
        api_path = utils.format_url('/v1/sys/leases/revoke-prefix/{prefix}', prefix=prefix)
        return self._adapter.put(
            url=api_path,
            json=params,
        )

    def revoke_force(self, prefix):
        """Revoke all secrets or tokens generated under a given prefix immediately.

        Unlike revoke_prefix, this path ignores backend errors encountered during revocation. This is potentially very
        dangerous and should only be used in specific emergency situations where errors in the backend or the connected
        backend service prevent normal revocation.

        Supported methods:
            PUT: /sys/leases/revoke-force/{prefix}. Produces: 204 (empty body)

        :param prefix: The prefix to revoke.
        :type prefix: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            'prefix': prefix,
        }
        api_path = utils.format_url('/v1/sys/leases/revoke-force/{prefix}', prefix=prefix)
        return self._adapter.put(
            url=api_path,
            json=params,
        )



#!/usr/bin/env python
from setuptools import setup, find_packages


def load_long_description():
    with open("README.md", "r") as fh:
        long_description = fh.read()
    return long_description


setup(
    name='hvac',
    version='0.10.7',
    description='HashiCorp Vault API client',
    long_description=load_long_description(),
    long_description_content_type="text/markdown",
    author='Ian Unruh <ianunruh@gmail.com>, Jeffrey Hogan <jeff.hogan1@gmail.com>',
    author_email='admin@python-hvac.org',
    url='https://github.com/hvac/hvac',
    keywords=['hashicorp', 'vault'],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=find_packages(exclude=['docs*', 'tests*']),
    install_requires=[
        'requests>=2.21.0',
        'six>=1.5.0',
    ],
    include_package_data=True,
    package_data={'hvac': ['version']},
    extras_require={
        'parser': ['pyhcl>=0.3.10']
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Gcp methods module."""
import json
import logging

from hvac import exceptions, utils
from hvac.api.vault_api_base import VaultApiBase
from hvac.constants.gcp import ALLOWED_SECRETS_TYPES, SERVICE_ACCOUNT_KEY_ALGORITHMS, SERVICE_ACCOUNT_KEY_TYPES

DEFAULT_MOUNT_POINT = 'gcp'


class Gcp(VaultApiBase):
    """Google Cloud Secrets Engine (API).

    Reference: https://www.vaultproject.io/api/secret/gcp/index.html
    """

    def configure(self, credentials=None, ttl=None, max_ttl=None, mount_point=DEFAULT_MOUNT_POINT):
        """Configure shared information for the Gcp secrets engine.

        Supported methods:
            POST: /{mount_point}/config. Produces: 204 (empty body)

        :param credentials: JSON credentials (either file contents or '@path/to/file') See docs for alternative ways to
            pass in to this parameter, as well as the required permissions.
        :type credentials: str | unicode
        :param ttl: – Specifies default config TTL for long-lived credentials (i.e. service account keys). Accepts
            integer number of seconds or Go duration format string.
        :type ttl: int | str
        :param max_ttl: Specifies the maximum config TTL for long-lived credentials (i.e. service account keys). Accepts
            integer number of seconds or Go duration format string.**
        :type max_ttl: int | str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = utils.remove_nones({
            'credentials': credentials,
            'ttl': ttl,
            'max_ttl': max_ttl,
        })
        api_path = utils.format_url('/v1/{mount_point}/config', mount_point=mount_point)
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_config(self, mount_point=DEFAULT_MOUNT_POINT):
        """Read the configured shared information for the Gcp secrets engine.

        Credentials will be omitted from returned data.

        Supported methods:
            GET: /{mount_point}/config. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url('/v1/{mount_point}/config', mount_point=mount_point)
        return self._adapter.get(
            url=api_path,
        )

    def create_or_update_roleset(self, name, project, bindings, secret_type=None, token_scopes=None,
                                 mount_point=DEFAULT_MOUNT_POINT):
        """Create a roleset or update an existing roleset.

        See roleset docs for the GCP secrets backend to learn more about what happens when you create or update a
            roleset.

        Supported methods:
            POST: /{mount_point}/roleset/{name}. Produces: 204 (empty body)

        :param name: Name of the role. Cannot be updated.
        :type name: str | unicode
        :param project: Name of the GCP project that this roleset's service account will belong to. Cannot be updated.
        :type project: str | unicode
        :param bindings: Bindings configuration string (expects HCL or JSON format in raw or base64-encoded string)
        :type bindings: str | unicode
        :param secret_type: Cannot be updated.
        :type secret_type: str | unicode
        :param token_scopes: List of OAuth scopes to assign to access_token secrets generated under this role set
            (access_token role sets only)
        :type token_scopes: list[str]
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        if secret_type is not None and secret_type not in ALLOWED_SECRETS_TYPES:
            error_msg = 'unsupported secret_type argument provided "{arg}", supported types: "{secret_type}"'
            raise exceptions.ParamValidationError(error_msg.format(
                arg=secret_type,
                secret_type=','.join(ALLOWED_SECRETS_TYPES),
            ))

        if isinstance(bindings, dict):
            bindings = json.dumps(bindings).replace(' ', '')
            logging.debug('bindings: %s' % bindings)

        params = {
            'project': project,
            'bindings': bindings,
        }
        params.update(
            utils.remove_nones({
                'secret_type': secret_type,
                'token_scopes': token_scopes,
            })
        )

        api_path = utils.format_url(
            '/v1/{mount_point}/roleset/{name}',
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def rotate_roleset_account(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the service account this roleset uses to generate secrets.

        This also replaces the key access_token roleset. This can be used to invalidate old secrets generated by the
            roleset or fix issues if a roleset's service account (and/or keys) was changed outside of Vault (i.e.
            through GCP APIs/cloud console).

        Supported methods:
            POST: /{mount_point}/roleset/{name}/rotate. Produces: 204 (empty body)

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            '/v1/{mount_point}/roleset/{name}/rotate',
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
        )

    def rotate_roleset_account_key(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the service account key this roleset uses to generate access tokens.

        This does not recreate the roleset service account.

        Supported methods:
            POST: /{mount_point}/roleset/{name}/rotate-key. Produces: 204 (empty body)

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            '/v1/{mount_point}/roleset/{name}/rotate-key',
            mount_point=mount_point,
            name=name
        )
        return self._adapter.post(
            url=api_path,
        )

    def read_roleset(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Read a roleset.

        Supported methods:
            GET: /{mount_point}/roleset/{name}. Produces: 200 application/json

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            '/v1/{mount_point}/roleset/{name}',
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_rolesets(self, mount_point=DEFAULT_MOUNT_POINT):
        """List configured rolesets.

        Supported methods:
            LIST: /{mount_point}/rolesets. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url('/v1/{mount_point}/rolesets', mount_point=mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_roleset(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing roleset by the given name.

        Supported methods:
            DELETE: /{mount_point}/roleset/{name} Produces: 200 application/json

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            '/v1/{mount_point}/roleset/{name}',
            name=name,
            mount_point=mount_point,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def generate_oauth2_access_token(self, roleset, mount_point=DEFAULT_MOUNT_POINT):
        """Generate an OAuth2 token with the scopes defined on the roleset.

        This OAuth access token can be used in GCP API calls, e.g. curl -H "Authorization: Bearer $TOKEN" ...

        Supported methods:
            GET: /{mount_point}/token/{roleset}. Produces: 200 application/json

        :param roleset: Name of an roleset with secret type access_token to generate access_token under.
        :type roleset: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            '/v1/{mount_point}/token/{roleset}',
            mount_point=mount_point,
            roleset=roleset,
        )
        return self._adapter.get(
            url=api_path,
        )

    def generate_service_account_key(self, roleset, key_algorithm='KEY_ALG_RSA_2048',
                                     key_type='TYPE_GOOGLE_CREDENTIALS_FILE', method='POST',
                                     mount_point=DEFAULT_MOUNT_POINT):
        """Generate Secret (IAM Service Account Creds): Service Account Key

        If using GET ('read'), the  optional parameters will be set to their defaults. Use POST if you want to specify
            different values for these params.

        :param roleset: Name of an roleset with secret type service_account_key to generate key under.
        :type roleset: str | unicode
        :param key_algorithm: Key algorithm used to generate key. Defaults to 2k RSA key You probably should not choose
            other values (i.e. 1k),
        :type key_algorithm: str | unicode
        :param key_type: Private key type to generate. Defaults to JSON credentials file.
        :type key_type: str | unicode
        :param method: Supported methods:
            POST: /{mount_point}/key/{roleset}. Produces: 200 application/json
            GET: /{mount_point}/key/{roleset}. Produces: 200 application/json
        :type method: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            '/v1/{mount_point}/key/{roleset}',
            mount_point=mount_point,
            roleset=roleset,
        )

        if method == 'POST':
            if key_algorithm not in SERVICE_ACCOUNT_KEY_ALGORITHMS:
                error_msg = 'unsupported key_algorithm argument provided "{arg}", supported algorithms: "{algorithms}"'
                raise exceptions.ParamValidationError(error_msg.format(
                    arg=key_algorithm,
                    algorithms=','.join(SERVICE_ACCOUNT_KEY_ALGORITHMS),
                ))
            if key_type not in SERVICE_ACCOUNT_KEY_TYPES:
                error_msg = 'unsupported key_type argument provided "{arg}", supported types: "{key_types}"'
                raise exceptions.ParamValidationError(error_msg.format(
                    arg=key_type,
                    key_types=','.join(SERVICE_ACCOUNT_KEY_TYPES),
                ))
            params = {
                'key_algorithm': key_algorithm,
                'key_type': key_type,
            }
            response = self._adapter.post(
                url=api_path,
                json=params,
            )

        elif method == 'GET':
            response = self._adapter.get(
                url=api_path,
            )

        else:
            error_message = '"method" parameter provided invalid value; POST or GET allowed, "{method}" provided'.format(method=method)
            raise exceptions.ParamValidationError(error_message)

        return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the APPROLE auth method."""

DEFAULT_MOUNT_POINT = 'approle'
ALLOWED_TOKEN_TYPES = ['service', 'batch', 'default', 'default-service', 'default-batch']
